@@ -27,10 +27,6 @@
  * See the ChangeLog files for a list of changes. 
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
 #include <glade/glade-xml.h>
 #include <libgnome/gnome-i18n.h>
 
@@ -38,15 +34,13 @@
 
 #include <string.h> /* For strlen (...) */
 
-#include <gedit-plugin.h>
-#include <gedit-debug.h>
-#include <gedit-menus.h>
-#include <gedit-utils.h>
+#include "gedit-plugin.h"
+#include "gedit-debug.h"
 
 #define MENU_ITEM_LABEL		N_("_Word Count")
-#define MENU_ITEM_PATH		"/menu/Tools/ToolsOps_2/"
+#define MENU_ITEM_PATH		"/menu/Search/SearchOps_3/"
 #define MENU_ITEM_NAME		"PluginWordCount"	
-#define MENU_ITEM_TIP		N_("Get info on current document")
+#define MENU_ITEM_TIP		N_("Get info on current document.")
 
 typedef struct _DocInfoDialog DocInfoDialog;
 
@@ -65,15 +59,9 @@ static void dialog_destroyed (GtkObject *obj,  void **dialog_pointer);
 static DocInfoDialog *get_dialog ();
 static void dialog_response_handler (GtkDialog *dlg, gint res_id,  DocInfoDialog *dialog);
 
-static void	word_count_real (void);
 static void	word_count_cb (BonoboUIComponent *uic, gpointer user_data, 
 			       const gchar* verbname);
-
-G_MODULE_EXPORT GeditPluginState update_ui (GeditPlugin *plugin, BonoboWindow *window);
-G_MODULE_EXPORT GeditPluginState activate (GeditPlugin *pd);
-G_MODULE_EXPORT GeditPluginState deactivate (GeditPlugin *pd);
-G_MODULE_EXPORT GeditPluginState init (GeditPlugin *pd);
-
+static void	word_count_real ();
 
 static void
 dialog_destroyed (GtkObject *obj,  void **dialog_pointer)
@@ -111,6 +99,7 @@ get_dialog ()
 	GladeXML *gui;
 	GtkWindow *window;
 	GtkWidget *content;
+	GtkWidget *button;
 
 	gedit_debug (DEBUG_PLUGINS, "");
 
@@ -118,13 +107,18 @@ get_dialog ()
 
 	if (dialog != NULL)
 	{
+		gdk_window_show (dialog->dialog->window);
+		gdk_window_raise (dialog->dialog->window);
+		gtk_widget_grab_focus (dialog->dialog);
+
 		gtk_window_set_transient_for (GTK_WINDOW (dialog->dialog),
 				window);
-
-		gtk_window_present (GTK_WINDOW (dialog->dialog));
-		
+	
 		gtk_widget_grab_focus (dialog->dialog);
 
+		if (!GTK_WIDGET_VISIBLE (dialog->dialog))
+			gtk_widget_show (dialog->dialog);
+
 		return dialog;
 	}
 
@@ -150,7 +144,7 @@ get_dialog ()
 
 	/* Add the update button */
 	gedit_dialog_add_button (GTK_DIALOG (dialog->dialog), 
-				 _("_Update"), GTK_STOCK_REFRESH, GTK_RESPONSE_OK);
+				 _("Update"), GTK_STOCK_REFRESH, GTK_RESPONSE_OK);
 
 	content			= glade_xml_get_widget (gui, "docinfo_dialog_content");
 
@@ -176,11 +170,11 @@ get_dialog ()
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog->dialog),
 					 GTK_RESPONSE_OK);
 
-	g_signal_connect (G_OBJECT (dialog->dialog), "destroy",
-			   G_CALLBACK (dialog_destroyed), &dialog);
+	gtk_signal_connect(GTK_OBJECT (dialog->dialog), "destroy",
+			   GTK_SIGNAL_FUNC (dialog_destroyed), &dialog);
 
-	g_signal_connect (G_OBJECT (dialog->dialog), "response",
-			   G_CALLBACK (dialog_response_handler), dialog);
+	gtk_signal_connect(GTK_OBJECT (dialog->dialog), "response",
+			   GTK_SIGNAL_FUNC (dialog_response_handler), dialog);
 
 	g_object_unref (gui);
 
@@ -201,7 +195,7 @@ word_count_cb (BonoboUIComponent *uic, gpointer user_data, const gchar* verbname
 }
 
 static void
-word_count_real (void)
+word_count_real ()
 {
 	DocInfoDialog *dialog;
 
@@ -232,7 +226,7 @@ word_count_real (void)
 		return;
 	}
 
-	text = gedit_document_get_chars (doc, 0, -1);
+	text = gedit_document_get_buffer (doc);
 	g_return_if_fail (g_utf8_validate (text, -1, NULL));
 
 	lines = gedit_document_get_line_count (doc);
@@ -303,20 +297,20 @@ word_count_real (void)
 G_MODULE_EXPORT GeditPluginState
 update_ui (GeditPlugin *plugin, BonoboWindow *window)
 {
-	BonoboUIComponent *uic;
+	BonoboUIEngine *ui_engine;
 	GeditDocument *doc;
 	
 	gedit_debug (DEBUG_PLUGINS, "");	
 	g_return_val_if_fail (window != NULL, PLUGIN_ERROR);
 
-	uic = gedit_get_ui_component_from_window (window);
+	ui_engine = bonobo_window_get_ui_engine (window);
 
 	doc = gedit_get_active_document ();
 
 	if (doc == NULL)		
-		gedit_menus_set_verb_sensitive (uic, "/commands/" MENU_ITEM_NAME, FALSE);
+		gedit_menus_set_verb_sensitive (ui_engine, "/commands/" MENU_ITEM_NAME, FALSE);
 	else
-		gedit_menus_set_verb_sensitive (uic, "/commands/" MENU_ITEM_NAME, TRUE);
+		gedit_menus_set_verb_sensitive (ui_engine, "/commands/" MENU_ITEM_NAME, TRUE);
 
 	return PLUGIN_OK;
 }
@@ -324,33 +318,80 @@ update_ui (GeditPlugin *plugin, BonoboWindow *window)
 G_MODULE_EXPORT GeditPluginState
 activate (GeditPlugin *pd)
 {
-	GList *top_windows;
-        gedit_debug (DEBUG_PLUGINS, "");
+	GList* top_windows;
+	
+	gedit_debug (DEBUG_PLUGINS, "");
 
-        top_windows = gedit_get_top_windows ();
-        g_return_val_if_fail (top_windows != NULL, PLUGIN_ERROR);
+	top_windows = gedit_get_top_windows ();
+	g_return_val_if_fail (top_windows != NULL, PLUGIN_ERROR);
+       
+	while (top_windows)
+	{
+		BonoboUIComponent* ui_component;
+		gchar *item_path;
+		ui_component = gedit_get_ui_component_from_window (
+					BONOBO_WINDOW (top_windows->data));
+		
+		item_path = g_strdup (MENU_ITEM_PATH MENU_ITEM_NAME);
 
-        while (top_windows)
-        {
-		gedit_menus_add_menu_item (BONOBO_WINDOW (top_windows->data),
-				     MENU_ITEM_PATH, MENU_ITEM_NAME,
-				     MENU_ITEM_LABEL, MENU_ITEM_TIP, NULL,
-				     word_count_cb);
+		if (!bonobo_ui_component_path_exists (ui_component, item_path, NULL))
+		{
+			gchar *xml;
+					
+			xml = g_strdup_printf ("<menuitem name=\"%s\" verb=\"\""
+					" _label=\"%s\" _tip=\"%s\" hidden=\"0\" />", 
+					MENU_ITEM_NAME, MENU_ITEM_LABEL, MENU_ITEM_TIP);
 
-                pd->update_ui (pd, BONOBO_WINDOW (top_windows->data));
+			bonobo_ui_component_set_translate (ui_component, 
+					MENU_ITEM_PATH, xml, NULL);
 
-                top_windows = g_list_next (top_windows);
-        }
+			bonobo_ui_component_set_translate (ui_component, 
+					"/commands/", "<cmd name = \"" MENU_ITEM_NAME "\" />", NULL);
 
-        return PLUGIN_OK;
+			bonobo_ui_component_add_verb (ui_component, 
+					MENU_ITEM_NAME, word_count_cb, 
+					      NULL); 
+
+			g_free (xml);
+		}
+		
+		g_free (item_path);
+
+		pd->update_ui (pd, BONOBO_WINDOW (top_windows->data));
+
+		top_windows = g_list_next (top_windows);
+	}
 }
 
 G_MODULE_EXPORT GeditPluginState
 deactivate (GeditPlugin *pd)
 {
-	gedit_menus_remove_menu_item_all (MENU_ITEM_PATH, MENU_ITEM_NAME);
+	GList* top_windows;
+	
+	gedit_debug (DEBUG_PLUGINS, "");
 
-	return PLUGIN_OK;
+	top_windows = gedit_get_top_windows ();
+	g_return_val_if_fail (top_windows != NULL, PLUGIN_ERROR);
+       
+	while (top_windows)
+	{
+		BonoboUIComponent* ui_component;
+		gchar *item_path;
+		ui_component = gedit_get_ui_component_from_window (
+					BONOBO_WINDOW (top_windows->data));
+		
+		item_path = g_strdup (MENU_ITEM_PATH MENU_ITEM_NAME);
+
+		if (bonobo_ui_component_path_exists (ui_component, item_path, NULL))
+		{
+			bonobo_ui_component_rm (ui_component, item_path, NULL);
+			bonobo_ui_component_rm (ui_component, "/commands/" MENU_ITEM_NAME, NULL);
+		}
+		
+		g_free (item_path);
+		
+		top_windows = g_list_next (top_windows);
+	}
 }
 
 G_MODULE_EXPORT GeditPluginState
