@@ -211,9 +211,6 @@ dialog_replace_get_dialog (void)
 		return NULL;
 	}
 
-	/* FIXME: remove when we will support case sensitive search */
-	gtk_widget_set_sensitive (dialog->case_sensitive, FALSE);
-
 	gtk_widget_show (dialog->replace_hbox);
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog->dialog)->vbox),
@@ -301,9 +298,6 @@ dialog_find_get_dialog (void)
 		return NULL;
 	}
 	
-	/* FIXME: remove when we will support case sensitive search */
-	gtk_widget_set_sensitive (dialog->case_sensitive, FALSE);
-	
 	gtk_widget_hide (replace_hbox);
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog->dialog)->vbox),
@@ -472,7 +466,7 @@ find_dlg_find_button_pressed (GeditDialogFind *dialog)
 				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_INFO,
 				GTK_BUTTONS_OK,
-				_("The string \"%s\" has not been found."), search_string);
+				_("The text \"%s\" was not found."), search_string);
 		
 		gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK);
 
@@ -543,7 +537,7 @@ replace_dlg_find_button_pressed (GeditDialogReplace *dialog)
 				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_INFO,
 				GTK_BUTTONS_OK,
-				_("The string \"%s\" has not been found."), search_string);
+				_("The text \"%s\" was not found."), search_string);
 		
 		gtk_dialog_set_default_response (GTK_DIALOG (message_dlg), GTK_RESPONSE_OK);
 
@@ -611,7 +605,7 @@ replace_dlg_replace_button_pressed (GeditDialogReplace *dialog)
 	if (strlen (replace_string) <= 0)
 		return;
 
-	selected_text = gedit_document_get_selected_text (doc);
+	selected_text = gedit_document_get_selected_text (doc, NULL, NULL);
 
 	if (strcmp (selected_text, search_string) != 0)
 	{
@@ -700,7 +694,7 @@ replace_dlg_replace_all_button_pressed (GeditDialogReplace *dialog)
 			GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 			GTK_MESSAGE_INFO,
 			GTK_BUTTONS_OK,
-			_("The string \"%s\" has not been found."), search_string);
+			_("The text \"%s\" was not found."), search_string);
 	}
 	else
 	{	
