@@ -47,16 +47,19 @@
 
 struct _GeditDocumentPrivate
 {
-	gchar*	uri;
-	gint 	untitled_number;	
+	gchar		*uri;
+	gint 		 untitled_number;	
 
-	gchar*  last_searched_text;
-	gchar*  last_replace_text;
-	gboolean  last_search_was_case_sensitive;
+	gchar		*last_searched_text;
+	gchar		*last_replace_text;
+	gboolean  	 last_search_was_case_sensitive;
+
+	guint		 auto_save_timeout;
+	gboolean	 last_save_was_manually; 
 	
-	gboolean readonly;
+	gboolean 	 readonly;
 
-	GeditUndoManager* undo_manager;
+	GeditUndoManager *undo_manager;
 };
 
 static gint current_max_untitled_num = 0;
@@ -92,6 +95,8 @@ static void gedit_document_can_undo_handler (GeditUndoManager* um, gboolean can_
 
 static void gedit_document_can_redo_handler (GeditUndoManager* um, gboolean can_redo, 
 					     GeditDocument* doc);
+static gboolean gedit_document_auto_save (GeditDocument *doc, GError **error);
+static gboolean gedit_document_auto_save_timeout (GeditDocument *doc);
 
 static GtkTextBufferClass *parent_class 	= NULL;
 static guint document_signals[LAST_SIGNAL] 	= { 0 };
@@ -243,6 +248,8 @@ gedit_document_init (GeditDocument *document)
 
 	document->priv->readonly = FALSE;
 
+	document->priv->last_save_was_manually = TRUE;
+
 	document->priv->undo_manager = gedit_undo_manager_new (document);
 
 	g_signal_connect (G_OBJECT (document->priv->undo_manager), "can_undo",
@@ -268,6 +275,9 @@ gedit_document_finalize (GObject *object)
 
 	g_return_if_fail (document->priv != NULL);
 
+	if (document->priv->auto_save_timeout > 0)
+		g_source_remove (document->priv->auto_save_timeout);
+
 	if (document->priv->uri)
     	{
 		g_free (document->priv->uri);
@@ -288,6 +298,7 @@ gedit_document_finalize (GObject *object)
 	if (document->priv->last_replace_text)
 		g_free (document->priv->last_replace_text);
 
+
 	g_object_unref (G_OBJECT (document->priv->undo_manager));
 	
 	g_free (document->priv);
@@ -361,31 +372,37 @@ gedit_document_new_with_uri (const gchar *uri, GError **error)
 void 		
 gedit_document_set_readonly (GeditDocument *document, gboolean readonly)
 {
-	GtkTextIter start;
-	GtkTextIter end;
-
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_if_fail (document != NULL);
 	g_return_if_fail (document->priv != NULL);
 
-	if (document->priv->readonly == readonly) 
-		return;
-
-	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (document), &start, 0);
-      	gtk_text_buffer_get_end_iter (GTK_TEXT_BUFFER (document), &end);
-
-	if(readonly) 
+	if (readonly) 
 	{
-		gtk_text_buffer_apply_tag_by_name (GTK_TEXT_BUFFER (document), 
-					NOT_EDITABLE_TAG_NAME, &start, &end);
+		if (gedit_settings->auto_save && (document->priv->auto_save_timeout > 0))
+		{
+			gedit_debug (DEBUG_DOCUMENT, "Remove autosave timeout");
+
+			g_source_remove (document->priv->auto_save_timeout);
+			document->priv->auto_save_timeout = 0;
+		}
 	}
 	else
 	{
-		gtk_text_buffer_remove_tag_by_name (GTK_TEXT_BUFFER (document), 
-					NOT_EDITABLE_TAG_NAME, &start, &end);
+		if (gedit_settings->auto_save && (document->priv->auto_save_timeout <= 0))
+		{
+			gedit_debug (DEBUG_DOCUMENT, "Install autosave timeout");
+
+			document->priv->auto_save_timeout = g_timeout_add 
+				(gedit_settings->auto_save_interval*1000*60,
+		 		 (GSourceFunc)gedit_document_auto_save_timeout,
+		  		 document);		
+		}
 	}
 
+	if (document->priv->readonly == readonly) 
+		return;
+
 	document->priv->readonly = readonly;
 
 	g_signal_emit (G_OBJECT (document),
@@ -477,6 +494,48 @@ gedit_document_io_error_quark()
   return quark;
 }
 
+static gboolean
+gedit_document_auto_save_timeout (GeditDocument *doc)
+{
+	GError *error = NULL;
+
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_val_if_fail (!gedit_document_is_readonly (doc), FALSE);
+
+	/* Romove timeout if now gedit_settings->auto_save is FALSE */
+	if (!gedit_settings->auto_save) 
+		return FALSE;
+
+	if (!gedit_document_get_modified (doc))
+		return TRUE;
+
+	gedit_document_auto_save (doc, &error);
+
+	if (error) 
+	{
+		/* FIXME - Should we actually tell 
+		 * the user there was an error? - James */
+		g_error_free (error);
+		return TRUE;
+	}
+
+	return TRUE;
+}
+
+static gboolean
+gedit_document_auto_save (GeditDocument* doc, GError **error)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+	
+	g_return_val_if_fail (doc != NULL, FALSE);
+
+	if (gedit_document_save_as_real (doc, doc->priv->uri, doc->priv->last_save_was_manually, NULL))
+		doc->priv->last_save_was_manually = FALSE;
+
+	return TRUE;
+}
+
 /**
  * gedit_document_load:
  * @doc: a GeditDocument
@@ -495,13 +554,15 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
    	gsize file_size;
 	GtkTextIter iter, end;
 	
-	gedit_debug (DEBUG_DOCUMENT, "");
+	gedit_debug (DEBUG_DOCUMENT, "File to load: %s", uri);
 
 	g_return_val_if_fail (doc != NULL, FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
 
 	res = gnome_vfs_x_read_entire_file (uri, &file_size, &file_contents);
 
+	gedit_debug (DEBUG_DOCUMENT, "End reading %s (result: %s)", uri, gnome_vfs_result_to_string (res));
+	
 	if (res != GNOME_VFS_OK)
 	{
 		g_set_error (error, GEDIT_DOCUMENT_IO_ERROR, res,
@@ -579,7 +640,7 @@ gedit_document_load (GeditDocument* doc, const gchar *uri, GError **error)
 	gtk_text_buffer_set_modified (GTK_TEXT_BUFFER (doc), FALSE);
 
 	gedit_document_set_uri (doc, uri);
-	
+		
 	g_signal_emit (G_OBJECT (doc), document_signals[LOADED], 0);
 
 	return TRUE;
@@ -703,33 +764,76 @@ gedit_document_set_uri (GeditDocument* doc, const gchar* uri)
 gboolean 
 gedit_document_save (GeditDocument* doc, GError **error)
 {	
+	gboolean ret;
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_val_if_fail (doc != NULL, FALSE);
 	g_return_val_if_fail (doc->priv != NULL, FALSE);
 	g_return_val_if_fail (!doc->priv->readonly, FALSE);
 
-	return gedit_document_save_as_real (doc, doc->priv->uri, 
+	if (gedit_settings->auto_save) 
+	{
+		if (doc->priv->auto_save_timeout > 0)
+		{
+			g_source_remove (doc->priv->auto_save_timeout);
+			doc->priv->auto_save_timeout = 0;
+		}
+	}
+	
+	ret = gedit_document_save_as_real (doc, doc->priv->uri, 
 			gedit_settings->create_backup_copy, error);
+
+	if (ret)
+		doc->priv->last_save_was_manually = TRUE;
+
+	if (gedit_settings->auto_save && (doc->priv->auto_save_timeout <= 0)) 
+	{
+		doc->priv->auto_save_timeout =
+			g_timeout_add (gedit_settings->auto_save_interval*1000*60,
+				       (GSourceFunc)gedit_document_auto_save, doc);
+	}
+
+	return ret;
 }
 
 gboolean	
 gedit_document_save_as (GeditDocument* doc, const gchar *uri, GError **error)
 {	
+	gboolean ret = FALSE;
+	
 	gedit_debug (DEBUG_DOCUMENT, "");
 
 	g_return_val_if_fail (doc != NULL, FALSE);
 	g_return_val_if_fail (doc->priv != NULL, FALSE);
 	g_return_val_if_fail (uri != NULL, FALSE);
 
+	if (gedit_settings->auto_save) 
+	{
+
+		if (doc->priv->auto_save_timeout > 0)
+		{
+			g_source_remove (doc->priv->auto_save_timeout);
+			doc->priv->auto_save_timeout = 0;
+		}
+	}
+
 	if (gedit_document_save_as_real (doc, uri, FALSE, error))
 	{
 		gedit_document_set_uri (doc, uri);
 		gedit_document_set_readonly (doc, FALSE);
-		return TRUE;
+		doc->priv->last_save_was_manually = TRUE;
+
+		ret = TRUE;
 	}		
 
-	return FALSE;
+	if (gedit_settings->auto_save && (doc->priv->auto_save_timeout <= 0)) 
+	{
+		doc->priv->auto_save_timeout =
+			g_timeout_add (gedit_settings->auto_save_interval*1000*60,
+				       (GSourceFunc)gedit_document_auto_save, doc);
+	}
+
+	return ret;
 }
 
 static gboolean	
@@ -997,6 +1101,19 @@ gedit_document_insert_text (GeditDocument *doc, gint pos, const gchar *text, gin
 	gtk_text_buffer_insert (GTK_TEXT_BUFFER (doc), &iter, text, len);
 }
 
+void 
+gedit_document_insert_text_at_cursor (GeditDocument *doc, const gchar *text, gint len)
+{
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
+	g_return_if_fail (text != NULL);
+	g_return_if_fail (g_utf8_validate (text, len, NULL));
+
+	gtk_text_buffer_insert_at_cursor (GTK_TEXT_BUFFER (doc), text, len);
+}
+
+
 void 
 gedit_document_delete_text (GeditDocument *doc, gint start, gint end)
 {
@@ -1166,7 +1283,6 @@ gedit_document_goto_line (GeditDocument* doc, guint line)
 		line = line_count;
 
 	gtk_text_buffer_get_iter_at_line (GTK_TEXT_BUFFER (doc), &iter, line);
-
 	gtk_text_buffer_place_cursor (GTK_TEXT_BUFFER (doc), &iter);
 }
 
@@ -1208,13 +1324,10 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
 
 	search_flags = GTK_TEXT_SEARCH_VISIBLE_ONLY | GTK_TEXT_SEARCH_TEXT_ONLY;
 	
-	/* FIXME: enable support for case sensitive -- Paolo */
-	/*
 	if (!case_sensitive)
 	{
 		search_flags = search_flags | GTK_TEXT_SEARCH_CASE_INSENSITIVE;
 	}
-	*/
 
 	if (from_cursor)
 	{
@@ -1239,7 +1352,7 @@ gedit_document_find (GeditDocument* doc, const gchar* str,
     	{
       		GtkTextIter match_start, match_end;
 
-          	found = gtk_text_iter_forward_search (&iter, str, search_flags,
+          	found = gedit_text_iter_forward_search (&iter, str, search_flags,
                                         	&match_start, &match_end,
                                                	NULL);
 		if (found)
@@ -1285,7 +1398,7 @@ gedit_document_find_again (GeditDocument* doc)
 }
 
 gchar*
-gedit_document_get_selected_text (GeditDocument *doc)
+gedit_document_get_selected_text (GeditDocument *doc, gint *start, gint *end)
 {
 	GtkTextIter iter;
 	GtkTextIter sel_bound;
@@ -1303,7 +1416,13 @@ gedit_document_get_selected_text (GeditDocument *doc)
                                     &sel_bound,
                                     gtk_text_buffer_get_mark (GTK_TEXT_BUFFER (doc),
 					                      "selection_bound"));
-	gtk_text_iter_order (&sel_bound, &iter);	
+	gtk_text_iter_order (&iter, &sel_bound);	
+
+	if (start != NULL)
+		*start = gtk_text_iter_get_offset (&iter); 
+
+	if (end != NULL)
+		*end = gtk_text_iter_get_offset (&sel_bound); 
 
 	return gtk_text_buffer_get_slice (GTK_TEXT_BUFFER (doc), &iter, &sel_bound, TRUE);
 }
@@ -1400,3 +1519,17 @@ gedit_document_replace_all (GeditDocument *doc,
 	return cont;
 }
 
+guint
+gedit_document_get_line_at_offset (const GeditDocument *doc, guint offset)
+{
+	GtkTextIter iter;
+	
+	gedit_debug (DEBUG_DOCUMENT, "");
+
+	g_return_val_if_fail (GEDIT_IS_DOCUMENT (doc), 0);
+
+	gtk_text_buffer_get_iter_at_offset (GTK_TEXT_BUFFER (doc), &iter, offset);
+	
+	return gtk_text_iter_get_line (&iter);
+}
+
