@@ -64,6 +64,9 @@ static void gedit_view_cursor_moved 	(GtkTextBuffer     *buffer,
 					 GtkTextMark       *mark,
 					 gpointer           data);
 static void gedit_view_update_overwrite_mode_statusbar (GtkTextView* w, GeditView* view);
+static void gedit_view_doc_readonly_changed_handler (GeditDocument *document, 
+						     gboolean readonly, 
+						     GeditView *view);
 
 static GtkVBoxClass *parent_class = NULL;
 
@@ -111,11 +114,25 @@ gedit_view_grab_focus (GtkWidget *widget)
 
 	/* FIXME: Remove this dirty hack to have the cursor visible when we will 
 	 * understand why it is not visible */
+	
 	GTK_WIDGET_SET_FLAGS (GTK_WIDGET (view->priv->text_view), GTK_HAS_FOCUS);
 	g_object_set (G_OBJECT (view->priv->text_view), "cursor_visible", FALSE, NULL);
 	g_object_set (G_OBJECT (view->priv->text_view), "cursor_visible", TRUE, NULL);
+	
 }
 
+static void
+gedit_view_doc_readonly_changed_handler (GeditDocument *document, gboolean readonly,
+		GeditView *view)
+{
+	gedit_debug (DEBUG_VIEW, "");
+
+	g_return_if_fail (GEDIT_IS_VIEW (view));
+
+	gtk_text_view_set_editable (view->priv->text_view, !readonly);	
+}
+
+
 static void
 gedit_view_class_init (GeditViewClass *klass)
 {
@@ -140,7 +157,8 @@ gedit_view_get_lines (GtkTextView  *text_view,
 {
 	GtkTextIter iter;
 	gint count;
-	gint size;  
+	gint size;
+      	gint last_line_num;	
 
 	gedit_debug (DEBUG_VIEW, "");
 
@@ -159,13 +177,12 @@ gedit_view_get_lines (GtkTextView  *text_view,
   	while (!gtk_text_iter_is_end (&iter))
     	{
 		gint y, height;
-		gint line_num;
       
 		gtk_text_view_get_line_yrange (text_view, &iter, &y, &height);
 
 		g_array_append_val (buffer_coords, y);
-		line_num = gtk_text_iter_get_line (&iter);
-		g_array_append_val (numbers, line_num);
+		last_line_num = gtk_text_iter_get_line (&iter);
+		g_array_append_val (numbers, last_line_num);
       	
 		++count;
 
@@ -175,6 +192,23 @@ gedit_view_get_lines (GtkTextView  *text_view,
 		gtk_text_iter_forward_line (&iter);
 	}
 
+	if (gtk_text_iter_is_end (&iter))
+    	{
+		gint y, height;
+		gint line_num;
+      
+		gtk_text_view_get_line_yrange (text_view, &iter, &y, &height);
+
+		line_num = gtk_text_iter_get_line (&iter);
+
+		if (line_num != last_line_num)
+		{
+			g_array_append_val (buffer_coords, y);
+			g_array_append_val (numbers, line_num);
+			++count;
+		}
+	}
+
 	*countp = count;
 }
 
@@ -201,8 +235,9 @@ gedit_view_line_numbers_expose (GtkWidget      *widget,
   	gint layout_width;
 	gchar *str;
 
-	gedit_debug (DEBUG_VIEW, "");
-
+	/* It is annoying -- Paolo
+	 * gedit_debug (DEBUG_VIEW, "");
+	 */
 	text_view = GTK_TEXT_VIEW (widget);
   
 	/* See if this expose is on the line numbers window */
@@ -474,7 +509,14 @@ gedit_view_new (GeditDocument *doc)
 			  "toggle_overwrite",/* cursor moved */
 			  G_CALLBACK (gedit_view_update_overwrite_mode_statusbar),
 			  view);
-	
+
+	g_signal_connect (doc,
+			  "readonly_changed",
+			  G_CALLBACK (gedit_view_doc_readonly_changed_handler),
+			  view);
+
+	gtk_text_view_set_editable (view->priv->text_view, !gedit_document_is_readonly (doc));	
+
 	gedit_debug (DEBUG_VIEW, "END");
 
 	return view;
@@ -735,16 +777,13 @@ gedit_view_set_cursor_position_statusbar (GeditView *view, GtkWidget* status)
 	gedit_debug (DEBUG_VIEW, "");
 
 	g_return_if_fail (GEDIT_IS_VIEW (view));
-	g_return_if_fail ((status == NULL) || (GTK_IS_STATUSBAR (status)));
 	
 	view->priv->cursor_position_statusbar = status;
 
 	if ((status != NULL) && (view->priv->document != NULL))
-	{
 		gedit_view_update_cursor_position_statusbar
 			(GTK_TEXT_BUFFER (view->priv->document),		
 			 view);
-	}
 }
 
 void
@@ -753,7 +792,6 @@ gedit_view_set_overwrite_mode_statusbar (GeditView *view, GtkWidget* status)
 	gedit_debug (DEBUG_VIEW, "");
 
 	g_return_if_fail (GEDIT_IS_VIEW (view));
-	g_return_if_fail ((status == NULL) || (GTK_IS_STATUSBAR (status)));
 	
 	view->priv->overwrite_mode_statusbar = status;
 
